#light
namespace EeekSoft.FSharp

(********************************* PARALLEL **********************************)

/// Module for performing standard list operations in parallel.
/// Implements filter and map functions.
module ParallelList : begin

  /// Returns current number of threads used for executing operations.
  /// The default value is 2
  val get_thread_count : unit -> int
  
  /// Sets number of threads to be used for executing operations.
  val set_thread_count : int -> unit
  
  /// Return a new collection containing only the elements of the collection for 
  /// which the given predicate returns "true". Predicates are executed in 
  /// parallel and thus shouldn't have any side-effects!
  val filter : ('a -> bool) -> 'a list -> 'a list
  
  /// Build a new collection whose elements are the results of applying the 
  /// given function to each of the elements of the collection. Functions are 
  /// executed in parallel and thus shouldn't have any side-effects!
  val map : ('a -> 'b) -> 'a list -> 'b list
end